@@ -29,6 +29,7 @@ from ..common.models import (
     EventType,
     MatterNodeData,
     MatterNodeEvent,
+    MatterSoftwareVersion,
     MessageType,
     NodePingResult,
     ResultMessageBase,
@@ -37,7 +38,7 @@ from ..common.models import (
     SuccessResultMessage,
 )
 from .connection import MatterClientConnection
-from .exceptions import ConnectionClosed, InvalidServerVersion, InvalidState
+from .exceptions import ConnectionClosed, InvalidState, ServerVersionTooOld
 from .models.node import (
     MatterFabricData,
     MatterNode,
@@ -130,6 +131,12 @@ class MatterClient:
             return node
         raise NodeNotExists(f"Node {node_id} does not exist or is not yet interviewed")
 
+    async def set_default_fabric_label(self, label: str | None) -> None:
+        """Set the default fabric label."""
+        await self.send_command(
+            APICommand.SET_DEFAULT_FABRIC_LABEL, require_schema=11, label=label
+        )
+
     async def commission_with_code(
         self, code: str, network_only: bool = False
     ) -> MatterNodeData:
@@ -181,7 +188,7 @@ class MatterClient:
     async def open_commissioning_window(
         self,
         node_id: int,
-        timeout: int = 300,
+        timeout: int = 300,  # noqa: ASYNC109 timeout parameter required for native timeout
         iteration: int = 1000,
         option: int = 1,
         discriminator: Optional[int] = None,
@@ -273,14 +280,14 @@ class MatterClient:
     async def get_node_ip_addresses(
         self, node_id: int, prefer_cache: bool = True, scoped: bool = False
     ) -> list[str]:
-        """Return the currently known (scoped) IP-adress(es)."""
+        """Return the currently known (scoped) IP-address(es)."""
         if TYPE_CHECKING:
             assert self.server_info is not None
         if self.server_info.schema_version >= 8:
             return cast(
                 list[str],
                 await self.send_command(
-                    APICommand.GET_NODE_IP_ADRESSES,
+                    APICommand.GET_NODE_IP_ADDRESSES,
                     require_schema=8,
                     node_id=node_id,
                     prefer_cache=prefer_cache,
@@ -509,14 +516,43 @@ class MatterClient:
         """Interview a node."""
         await self.send_command(APICommand.INTERVIEW_NODE, node_id=node_id)
 
-    async def send_command(
+    async def check_node_update(self, node_id: int) -> MatterSoftwareVersion | None:
+        """Check Node for updates.
+
+        Return a dict with the available update information. Most notable
+        "softwareVersion" contains the integer value of the update version which then
+        can be used for the update_node command to trigger the update.
+
+        The "softwareVersionString" is a human friendly version string.
+        """
+        data = await self.send_command(
+            APICommand.CHECK_NODE_UPDATE, node_id=node_id, require_schema=10
+        )
+        if data is None:
+            return None
+
+        return dataclass_from_dict(MatterSoftwareVersion, data)
+
+    async def update_node(
+        self,
+        node_id: int,
+        software_version: int | str,
+    ) -> None:
+        """Start node update to a particular version."""
+        await self.send_command(
+            APICommand.UPDATE_NODE,
+            node_id=node_id,
+            software_version=software_version,
+            require_schema=10,
+        )
+
+    def _prepare_message(
         self,
         command: str,
         require_schema: int | None = None,
         **kwargs: Any,
-    ) -> Any:
-        """Send a command and get a response."""
-        if not self.connection.connected or not self._loop:
+    ) -> CommandMessage:
+        if not self.connection.connected:
             raise InvalidState("Not connected")
 
         if (
@@ -524,16 +560,28 @@ class MatterClient:
             and self.server_info is not None
             and require_schema > self.server_info.schema_version
         ):
-            raise InvalidServerVersion(
+            raise ServerVersionTooOld(
                 "Command not available due to incompatible server version. Update the Matter "
-                f"Server to a version that supports at least api schema {require_schema}."
+                f"Server to a version that supports at least api schema {require_schema}.",
             )
 
-        message = CommandMessage(
+        return CommandMessage(
             message_id=uuid.uuid4().hex,
             command=command,
             args=kwargs,
         )
+
+    async def send_command(
+        self,
+        command: str,
+        require_schema: int | None = None,
+        **kwargs: Any,
+    ) -> Any:
+        """Send a command and get a response."""
+        if not self._loop:
+            raise InvalidState("Not connected")
+
+        message = self._prepare_message(command, require_schema, **kwargs)
         future: asyncio.Future[Any] = self._loop.create_future()
         self._result_futures[message.message_id] = future
         await self.connection.send_message(message)
@@ -549,22 +597,8 @@ class MatterClient:
         **kwargs: Any,
     ) -> None:
         """Send a command without waiting for the response."""
-        if not self.server_info:
-            raise InvalidState("Not connected")
 
-        if (
-            require_schema is not None
-            and require_schema > self.server_info.schema_version
-        ):
-            raise InvalidServerVersion(
-                "Command not available due to incompatible server version. Update the Matter "
-                f"Server to a version that supports at least api schema {require_schema}."
-            )
-        message = CommandMessage(
-            message_id=uuid.uuid4().hex,
-            command=command,
-            args=kwargs,
-        )
+        message = self._prepare_message(command, require_schema, **kwargs)
         await self.connection.send_message(message)
 
     async def get_diagnostics(self) -> ServerDiagnostics:
@@ -578,6 +612,8 @@ class MatterClient:
         if self.connection.connected:
             # already connected
             return
+
+        self._stop_called = False
         # NOTE: connect will raise when connecting failed
         await self.connection.connect()
 
